/**
 * Manages a map of subscriptions keyed on a number ID.
 *
 * @internal
 */
class SubscriptionMap {
    map = new Map();
    /**
     * Adds a new subscription with the given {@link id}.
     *
     * @param id - A unique identifier for this subscription among all other entries in this map.
     * @param subscription - The subscription, unsubscribed when {@link SubscriptionMap#remove} is called.
     * @throws If there already exists an entry with the given {@link id}.
     */
    add(id, subscription) {
        if (this.map.has(id)) {
            throw new Error(`subscription already exists with ID ${id}`);
        }
        this.map.set(id, subscription);
    }
    /**
     * Unsubscribes the subscription that was previously added with the given {@link id}, and removes it from the
     * map.
     */
    remove(id) {
        const subscription = this.map.get(id);
        if (!subscription) {
            throw new Error(`no subscription with ID ${id}`);
        }
        try {
            subscription.unsubscribe();
        }
        finally {
            this.map.delete(id);
        }
    }
    /**
     * Unsubscribes all subscriptions in this map and clears it.
     */
    unsubscribe() {
        try {
            for (const subscription of this.map.values()) {
                subscription.unsubscribe();
            }
        }
        finally {
            this.map.clear();
        }
    }
}

export { SubscriptionMap };
//# sourceMappingURL=common.js.map
