import { from, throwError, of } from 'rxjs';
import { map, distinctUntilChanged, switchMap, catchError, publishReplay, refCount, tap } from 'rxjs/operators/index.js';
import { ExtensionsByLimitDocument } from '../components/generated.js';
import { createAggregateError } from '../util/errors.js';
import { isErrorLike, asError } from '@adminide-stack/extension-api';
import { parseExtensionManifestOrError } from './extension-manifest.js';
import { isEqual } from 'lodash-es';

/**
 * @returns An observable that emits the list of extensions configured in the viewer's final settings upon
 * subscription and each time it changes.
 */
function viewerConfiguredExtensions({ settings, requestGraphQL, }) {
    return from(settings).pipe(map(settings => extensionIDsFromSettings(settings)), distinctUntilChanged((a, b) => isEqual(a, b)), switchMap(extensionIDs => queryConfiguredRegistryExtensions({ requestGraphQL }, extensionIDs)), catchError(error => throwError(asError(error))), publishReplay(), refCount());
}
/**
 * Query the GraphQL API for registry metadata about the extensions given in {@link extensionIDs}.
 *
 * @return An observable that emits once with the results.
 */
function queryConfiguredRegistryExtensions({ requestGraphQL }, extensionIDs) {
    if (extensionIDs.length === 0) {
        return of([]);
    }
    const variables = {
        first: extensionIDs.length,
        prioritizeExtensionIDs: extensionIDs,
    };
    return requestGraphQL(ExtensionsByLimitDocument, variables, false).pipe(tap(({ data }) => console.log("[TAP.DATA]:", data)), map(({ data, errors }) => {
        if (!data?.extensions?.nodes) {
            throw createAggregateError(errors);
        }
        return data.extensions.nodes.map(({ id, extensionID, url, manifest, viewerCanAdminister }) => ({
            id,
            extensionID,
            url,
            manifest: manifest ? { raw: manifest.raw } : null,
            viewerCanAdminister,
        }));
    }), map(registryExtensions => {
        const configuredExtensions = [];
        for (const extensionID of extensionIDs) {
            const registryExtension = registryExtensions.find(extension => extension.extensionID === extensionID);
            configuredExtensions.push(registryExtension
                ? toConfiguredRegistryExtension(registryExtension)
                : { id: extensionID, manifest: null, rawManifest: null, registryExtension: undefined });
        }
        return configuredExtensions;
    }));
}
/**
 * Converts to a {@link IConfiguredRegistryExtension} value.
 *
 * @template X the extension type
 */
function toConfiguredRegistryExtension(extension) {
    return {
        id: extension.extensionID,
        manifest: extension.manifest ? parseExtensionManifestOrError(extension.manifest.raw) : null,
        rawManifest: (extension && extension.manifest && extension.manifest.raw) || null,
        registryExtension: extension,
    };
}
/** Reports whether the given extension is enabled in the settings. */
function isExtensionEnabled(settings, extensionID) {
    return !!settings && !isErrorLike(settings) && !!settings.extensions && !!settings.extensions[extensionID];
}
/**
 * Returns the extension's script URL from its manifest.
 *
 * @param extension The extension whose script URL to get.
 * @throws If the script URL can't be determined.
 * @returns The extension's script URL.
 */
function getScriptURLFromExtensionManifest(extension) {
    if (!extension.registryExtension) {
        throw new Error(`extension ${JSON.stringify(extension.id)}: no manifest found`);
    }
    if (isErrorLike(extension.registryExtension)) ;
    if (!extension.registryExtension.url) {
        throw new Error(`extension ${JSON.stringify(extension.id)}: no "bundleURL" property in manifest`);
    }
    return extension.registryExtension.url;
}
/**
 * @throws An error if the final settings has an error.
 * @returns An array of extension IDs configured in the settings.
 */
function extensionIDsFromSettings(settings) {
    if (isErrorLike(settings)) {
        throw asError(settings);
    }
    if (!settings || !(settings).extensions) {
        return [];
    }
    return Object.keys((settings).extensions);
}

export { extensionIDsFromSettings, getScriptURLFromExtensionManifest, isExtensionEnabled, queryConfiguredRegistryExtensions, toConfiguredRegistryExtension, viewerConfiguredExtensions };
//# sourceMappingURL=extension.js.map
